/*****************************************************************************
 * HTML Filter                                                               *
 * ========================================================================= *
 * This code uses Jsoup, a Java HTML parser, available at http://jsoup.org/  *
 *****************************************************************************/

import java.io.*;
import java.util.*;

import org.jsoup.*;
import org.jsoup.nodes.*;
import org.jsoup.safety.*;
import org.jsoup.select.*;

class HtmlFilter {

    private static boolean DEBUGGING = false;
    
    /** Utility tool to add two random spaces in the middle of a hyperlink. */
    private static String addRandomSpaces(String str) {
        Random generator = new Random();
        int len = str.length();
        
        if (len <= 0) { return str; }
        
        int randPos = generator.nextInt(len);
        String newStr = str.substring(0, randPos) + " " + str.substring(randPos);
        randPos = generator.nextInt(len+1);
        return newStr.substring(0, randPos) + " " + newStr.substring(randPos);
    }
    
    /** Replace every "img" tag with the message "[IMAGE <number>]".
     *  The "src" attributes are collected and then appended to the bottom of
     *  the document, much like references are collected at the end of an
     *  academic paper. However, the "src" attributes are not displayed as
     *  hyperlinks: the raw text of the "src" is displayed instead.
     *  The message in the body is thus a hyperlink to the corresponding
     *  reference anchor at the bottom of the document.
     *
     *  We know, because of the sanitization pre-processing, that the only
     *  possible "src" attributes begin with "http", "https", "ftp", and
     *  "mailto". */ 
    private static void replaceImgTags(Document doc) {
        LinkedList<String> imgSrcs = new LinkedList<String>();
        Elements imgs = doc.getElementsByTag("img");
        int counter = 1;
        
        for (Element img : imgs) {
            // Collect the sources.
            imgSrcs.add(img.attr("src"));
            // Add a hyperlink node in the DOM, linked to a reference
            // anchor at the bottom of the document.
            Element newAElmt = doc.createElement("a");
            newAElmt.attr("href", "#__img" + counter);
            newAElmt.html("[IMAGE " + counter + "]");
            Element breakElmt = doc.createElement("br");
            
            img.before(newAElmt);
            newAElmt.after(breakElmt);
            counter++;
        }
        
        // Remove all the "img" tags.
        imgs.remove();
        
        counter = 1;
        Element body = doc.body();
        Element breakElmt = doc.createElement("br");
        body.appendChild(breakElmt);
        breakElmt = doc.createElement("br");
        body.appendChild(breakElmt);
        
        if (imgSrcs.size() > 0) {
            Element boldElmt = doc.createElement("strong");
            boldElmt.html("Images");
            body.appendChild(boldElmt);
            breakElmt = doc.createElement("br");
            body.appendChild(breakElmt);
        }
        
        // Append the sources to the end of the body of the file.
        for (String src : imgSrcs)
        {
            String reference = "[" + counter + "]: " + addRandomSpaces(src);
            Element newAElmt = doc.createElement("a");
            newAElmt.attr("name", "__img" + counter);
            newAElmt.html(reference);
            breakElmt = doc.createElement("br");
            
            body.appendChild(newAElmt);
            body.appendChild(breakElmt);
            counter++;
        }
    }
    
    /** Works similar to replaceImgTags, but is used to replace
     *  "a" tags instead. */
    private static void replaceATags(Document doc) {
        LinkedList<String> aSrcs = new LinkedList<String>();
        Elements hyperlinks = doc.getElementsByTag("a");
        int counter = 1;
        
        for (Element hyperlink : hyperlinks) {
            // Collect the sources.
            aSrcs.add(hyperlink.attr("href"));
            // Add a hyperlink node in the DOM, linked to a reference
            // anchor at the bottom of the document.
            Element newAElmt = doc.createElement("a");
            newAElmt.attr("href", "#__link" + counter);
            newAElmt.html("[LINK " + counter + "]");
            Element breakElmt = doc.createElement("br");
            
            hyperlink.before(newAElmt);
            newAElmt.after(breakElmt);
            counter++;
        }
        
        // Remove all the "a" tags.
        hyperlinks.remove();
        
        counter = 1;
        Element body = doc.body();
        Element breakElmt = doc.createElement("br");
        body.appendChild(breakElmt);
        breakElmt = doc.createElement("br");
        body.appendChild(breakElmt);
        
        if (aSrcs.size() > 0) {
            Element boldElmt = doc.createElement("strong");
            boldElmt.html("Links");
            body.appendChild(boldElmt);
            breakElmt = doc.createElement("br");
            body.appendChild(breakElmt);
        }
        
        // Append the sources to the end of the body of the file.
        for (String src : aSrcs)
        {
            String reference = "[" + counter + "]: " + addRandomSpaces(src);
            Element newAElmt = doc.createElement("a");
            newAElmt.attr("name", "__link" + counter);
            newAElmt.html(reference);
            breakElmt = doc.createElement("br");
            
            body.appendChild(newAElmt);
            body.appendChild(breakElmt);
            counter++;
        }
    }
    
    public static void main(String[] args) {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        String html = "", newHtml;
        
        try {
            while ((newHtml = in.readLine()) != null) {
                html += "\n" + newHtml;
            }
            
            if (DEBUGGING) {
                System.out.println("HTMLFITLER-DEBUG: Contents: \n" + html);
            }
        } catch (IOException e) {
            System.out.println("HTMLFILTER: Error reading from standard input.");
            System.out.println("Exiting...");
            System.exit(1);
        }
        
        /* Use Jsoup's existing sanitization scheme and a predefined whitelist,
         * as detailed at
         * http://jsoup.org/cookbook/cleaning-html/whitelist-sanitizer, to
         * preserve a certain list of tags. */
         
        /* The whitelist below allows the following tags:
         * a, b, blockquote, br, cite, code, dd, dl, dt, em, i, li, ol, p,
         * pre, q, small, strike, strong, sub, sup, u, ul.
         * It also allows "img" tags.
         *
         * As per the documentation
         * http://jsoup.org/apidocs/org/jsoup/safety/Whitelist.html#basicWithImages()
         * links ("a" elements) can point to "http", "https", "ftp", "mailto", and
         * have an enforced "rel=nofollow" attribute.
         */
        Whitelist whitelist = Whitelist.basicWithImages();
        
        /* Add a few more tags to the whitelist. */
        whitelist.addTags("tt", "big", "h1", "h2", "h3", "h4", "h5", "h6");
        
        String safer = Jsoup.clean(html, whitelist);
        Document htmlDoc = Jsoup.parse(safer);
        replaceATags(htmlDoc);
        replaceImgTags(htmlDoc);
        
        System.out.println(htmlDoc);
        System.exit(0);
    }
}
